/*
  mmlib.h
  Interface library for mergemod & mergemem
  
  This file is part of mergemem by Philipp Richter & Philipp Reisner

  mergemem is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  mergemem is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with mergemem; see the file COPYING.  If not, write to
  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/


/* 

****
 !!COMMENTS WELCOME!!
****

        mergemem library 

The intention of this file is to provide a _stable_ interface for
both, the the mergemod programmers _and_ the mergemem programmers. 
The goals of this interface are:

* Hide the interface of mergemod
  (device & ioctls, syscalls, proc fs hacks, ... )
* Hide the semantics of mergemod
  (one merge per call, manny merges per call
* Hide the page size
* Allow different checksums
* See this calls not as commans, see them as advices to the 
  memory management subsystem of the kernel
* Not to determine the search heuristics.


*/
#include <sys/types.h>
#include <sys/wait.h>


struct _mpinfo
{
  unsigned long v_addr;  /* Virtual adress          */
  unsigned long p_addr;  /* Physical adress         */
  unsigned long hash;    /* Hash                    */ 
  int count;             /* Reference count of page */
};

typedef struct _mpinfo Result;


size_t mgetphys(  
		pid_t pid,          
		unsigned long from,
		unsigned long to,
		Result* results,
		size_t n_results,
		unsigned long* cont_at );

/* 

getphys() writes to  v_addr, p_addr and count of the results array. 
It examines the memory of the process specified by pid in the range of from to
to and it fills up to n_results elements of results. 
If no error occours during execution it also sets cont_at to the lowest not 
jet examined page in the process.

RETURN VALUE

On success the number of filled entries is returned. It is not an error if this
number is smaller than n_results. (This may occout of various reasons)

ERRORS

    EFAULT  buf is outside your accessible address space.   
    ??????  pid is no valid process.
    ...

*/

size_t mgethash(
		pid_t pid,
		Result* results,
		size_t n_results,
		unsigned long (*hash_fun)(void*,size_t) );

unsigned long mh_addrot(void*,size_t);
unsigned long (*mh_best_known)(void*,size_t) = mh_addrot;

/*

gethash() fills the hash fields of the entries. You may dissable entries
from the array by setting the v_addr to -1. 

There are some predefined hash functions which may be implemented inside
the kernel thus causing less context switching overhead

  mh_best_known an alias to the best hash function of the following.
  mh_addrot     trivial machine word wide add and rotation of the first 
                half of a page.
  ... 

*/

struct _mmerge
{
  pid_t spid;
  unsigned long sv_addr;
  pid_t dpid;
  unsigned long dv_addr;
  int status;               /* holds the return code */
};

typedef struct _mmerge Merge;

size_t mmerge(
	      Merge* merges,
	      size_t size );

/*



*/








